@@ -201,7 +201,8 @@ class SelfAttention(nn.Module):
             mask = F.pad(mask, (mem_len + cmem_len, 0), value = True)
             dots.masked_fill_(~mask, mask_value)
 
-        mask = torch.ones(t, kv_len, **to(x)).triu_(diagonal = 1 + kv_len).bool()
+        total_mem_len = mem_len + cmem_len
+        mask = torch.ones(t, t + total_mem_len, **to(x)).triu_(diagonal = 1 + total_mem_len).bool()
         dots.masked_fill_(mask[None, None, ...], mask_value)
 
         attn = dots.softmax(dim=-1)
